@@ -266,161 +266,18 @@ char *cgi_url_unescape (char *value)
 
 NEOERR *cgi_js_escape (const char *in, char **esc)
 {
-  int nl = 0;
-  int l = 0;
-  unsigned char *buf = (unsigned char *)in;
-  unsigned char *s;
-
-  while (buf[l])
-  {
-    if (buf[l] == '/' || buf[l] == '"' || buf[l] == '\'' ||
-	buf[l] == '\\' || buf[l] == '>' || buf[l] == '<' || buf[l] < 32)
-    {
-      nl += 3;
-    }
-    nl++;
-    l++;
-  }
-
-  s = (unsigned char *) malloc (sizeof(unsigned char) * (nl + 1));
-  if (s == NULL)
-    return nerr_raise (NERR_NOMEM, "Unable to allocate memory to escape %s",
-	buf);
-
-  nl = 0; l = 0;
-  while (buf[l])
-  {
-    if (buf[l] == '/' || buf[l] == '"' || buf[l] == '\'' ||
-	buf[l] == '\\' || buf[l] == '>' || buf[l] == '<' || buf[l] < 32)
-    {
-      s[nl++] = '\\';
-      s[nl++] = 'x';
-      s[nl++] = "0123456789ABCDEF"[(buf[l] >> 4) & 0xF];
-      s[nl++] = "0123456789ABCDEF"[buf[l] & 0xF];
-      l++;
-    }
-    else
-    {
-      s[nl++] = buf[l++];
-    }
-  }
-  s[nl] = '\0';
-
-  *esc = (char *)s;
-  return STATUS_OK;
-}
-
-// List of all characters that must be escaped
-// List based on http://www.blooberry.com/indexdot/html/topics/urlencoding.htm
-static char EscapedChars[] = "$&+,/:;=?@ \"<>#%{}|\\^~[]`";
-
-// Check if a single character needs to be escaped
-static BOOL is_reserved_char(char c)
-{
-  int i = 0;
-
-  if (c < 32 || c > 122) {
-    return TRUE;
-  } else {
-    while (EscapedChars[i]) {
-      if (c == EscapedChars[i]) {
-        return TRUE;
-      }
-      ++i;
-    }
-  }
-  return FALSE;
-}
-
-NEOERR *cgi_url_escape_more (const char *in, char **esc,
-                             const char *other)
-{
-  int nl = 0;
-  int l = 0;
-  int x = 0;
-  unsigned char *buf = (unsigned char *)in;
-  unsigned char *uother = (unsigned char *)other;
-  unsigned char *s;
-  int match = 0;
-
-  while (buf[l])
-  {
-    if (is_reserved_char(buf[l]))
-    {
-      nl += 2;
-    }
-    else if (uother)
-    {
-      x = 0;
-      while (uother[x])
-      {
-	if (uother[x] == buf[l])
-	{
-	  nl +=2;
-	  break;
-	}
-	x++;
-      }
-    }
-    nl++;
-    l++;
-  }
-
-  s = (unsigned char *) malloc (sizeof(unsigned char) * (nl + 1));
-  if (s == NULL)
-    return nerr_raise (NERR_NOMEM, "Unable to allocate memory to escape %s",
-	buf);
-
-  nl = 0; l = 0;
-  while (buf[l])
-  {
-    match = 0;
-    if (buf[l] == ' ')
-    {
-      s[nl++] = '+';
-      l++;
-    }
-    else
-    {
-      if (is_reserved_char(buf[l]))
-      {
-	match = 1;
-      }
-      else if (uother)
-      {
-	x = 0;
-	while (uother[x])
-	{
-	  if (uother[x] == buf[l])
-	  {
-	    match = 1;
-	    break;
-	  }
-	  x++;
-	}
-      }
-      if (match)
-      {
-	s[nl++] = '%';
-	s[nl++] = "0123456789ABCDEF"[buf[l] / 16];
-	s[nl++] = "0123456789ABCDEF"[buf[l] % 16];
-	l++;
-      }
-      else
-      {
-	s[nl++] = buf[l++];
-      }
-    }
-  }
-  s[nl] = '\0';
-
-  *esc = (char *)s;
-  return STATUS_OK;
+  return nerr_pass(neos_js_escape(in, esc));
 }
 
 NEOERR *cgi_url_escape (const char *buf, char **esc)
 {
-  return nerr_pass(cgi_url_escape_more(buf, esc, NULL));
+  return nerr_pass(neos_url_escape(buf, esc, NULL));
+}
+
+NEOERR *cgi_url_escape_more (const char *in, char **esc,
+                                 const char *other)
+{
+  return nerr_pass(neos_url_escape(in, esc, other));
 }
 
 static NEOERR *_parse_query (CGI *cgi, char *query)
@@ -520,7 +377,7 @@ static NEOERR *_parse_post_form (CGI *cgi)
   l = hdf_get_value (cgi->hdf, "CGI.ContentLength", NULL);
   if (l == NULL) return STATUS_OK;
   len = atoi (l);
-  if (len == 0) return STATUS_OK;
+  if (len <= 0) return STATUS_OK;
 
   cgi->data_expected = len;
 
@@ -695,7 +552,8 @@ static NEOERR *cgi_pre_parse (CGI *cgi)
     char *d = hdf_get_value(cgi->hdf, "Query.debug_pause", NULL);
     char *d_p = hdf_get_value(cgi->hdf, "Config.DebugPassword", NULL);
 
-    if (d && d_p && !strcmp(d, d_p)) {
+    if (hdf_get_int_value(cgi->hdf, "Config.DebugEnabled", 0) &&
+        d && d_p && !strcmp(d, d_p)) {
       sleep(20);
     }
   }
@@ -732,6 +590,10 @@ NEOERR *cgi_register_parse_cb(CGI *cgi, const char *method, const char *ctype,
   my_pcb->ctype = strdup(ctype);
   if (my_pcb->method == NULL || my_pcb->ctype == NULL)
   {
+    if (my_pcb->method != NULL)
+      free(my_pcb->method);
+    if (my_pcb->ctype != NULL)
+      free(my_pcb->ctype);
     free(my_pcb);
     return nerr_raise(NERR_NOMEM, "Unable to allocate memory to register parse cb");
   }
@@ -826,6 +688,7 @@ NEOERR *cgi_parse (CGI *cgi)
     l = hdf_get_value (cgi->hdf, "CGI.ContentLength", NULL);
     if (l == NULL) return STATUS_OK;
     len = atoi (l);
+    if (len <= 0) return STATUS_OK;
 
     x = 0;
     while (x < len)
@@ -1199,7 +1062,8 @@ NEOERR *cgi_output (CGI *cgi, STRING *str)
 
   s = hdf_get_value (cgi->hdf, "Query.debug", NULL);
   e = hdf_get_value (cgi->hdf, "Config.DebugPassword", NULL);
-  if (s && e && !strcmp(s, e)) do_debug = 1;
+  if (hdf_get_int_value(cgi->hdf, "Config.DebugEnabled", 0) && 
+      s && e && !strcmp(s, e)) do_debug = 1;
   do_timefooter = hdf_get_int_value (cgi->hdf, "Config.TimeFooter", 1);
   ws_strip_level = hdf_get_int_value (cgi->hdf, "Config.WhiteSpaceStrip", 1);
 
@@ -1411,13 +1275,13 @@ NEOERR *cgi_register_strfuncs(CSPARSE *cs)
 {
   NEOERR *err;
 
-  err = cs_register_strfunc(cs, "url_escape", cgi_url_escape);
+  err = cs_register_esc_strfunc(cs, "url_escape", cgi_url_escape);
   if (err != STATUS_OK) return nerr_pass(err);
-  err = cs_register_strfunc(cs, "html_escape", cgi_html_escape_strfunc);
+  err = cs_register_esc_strfunc(cs, "html_escape", cgi_html_escape_strfunc);
   if (err != STATUS_OK) return nerr_pass(err);
   err = cs_register_strfunc(cs, "text_html", cgi_text_html_strfunc);
   if (err != STATUS_OK) return nerr_pass(err);
-  err = cs_register_strfunc(cs, "js_escape", cgi_js_escape);
+  err = cs_register_esc_strfunc(cs, "js_escape", cgi_js_escape);
   if (err != STATUS_OK) return nerr_pass(err);
   err = cs_register_strfunc(cs, "html_strip", cgi_html_strip_strfunc);
   if (err != STATUS_OK) return nerr_pass(err);
@@ -1455,7 +1319,8 @@ NEOERR *cgi_display (CGI *cgi, const char *cs_file)
 
   debug = hdf_get_value (cgi->hdf, "Query.debug", NULL);
   t = hdf_get_value (cgi->hdf, "Config.DumpPassword", NULL);
-  if (debug && t && !strcmp (debug, t)) do_dump = 1;
+  if (hdf_get_int_value(cgi->hdf, "Config.DebugEnabled", 0) &&
+      debug && t && !strcmp (debug, t)) do_dump = 1;
 
   do
   {
@@ -1567,7 +1432,7 @@ void cgi_vredirect (CGI *cgi, int uri, const char *fmt, va_list ap)
 
     host = hdf_get_value (cgi->hdf, "HTTP.Host", NULL);
     if (host == NULL)
-      host = hdf_get_value (cgi->hdf, "CGI.ServerName", NULL);
+      host = hdf_get_value (cgi->hdf, "CGI.ServerName", "localhost");
 
     cgiwrap_writef ("Location: %s://%s", https ? "https" : "http", host);
 
