@@ -157,3 +157,10 @@ JNIEXPORT jstring JNICALL Java_org_clearsilver_CS__1render
   return retval;
 }
 
+// Change global HDF
+JNIEXPORT void JNICALL Java_org_clearsilver_CS__1setGlobalHdf
+(JNIEnv *env, jobject objclass, jint cs_obj_ptr, jint hdf_obj_ptr) {
+  HDF *hdf = (HDF *)hdf_obj_ptr;
+  CSPARSE *cs = (CSPARSE *)cs_obj_ptr;
+  cs->global_hdf = hdf;
+}
